#include "building.h"
#include "ErrorHandler.h"

void Building::scan()
{
	ML_TRACE("Building::scan");

	const string filename = getIsFile();
	ifstream  is(filename.c_str());

	is.exceptions(ios_base::goodbit);
	try{
		is.exceptions(ios_base::badbit | ios_base::failbit);
		int n; is >> n;

		redim(n);
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		for(int i = 1; i<= getNoElms(); i++)
		{
			blg->scan(is); advance();
		}
		is.close();
	}
	catch(...)
	{
		cerr << ErrorBuilding_scan(filename);
		CERR << ErrorBuilding_scan(filename);
		exit(1);
	}
}

void Building::lists()
{
	ML_TRACE("Building::lists");

	CLST << endl;
	CLST << endl;
	CLST << "<< Scanned building data list >>" << endl;
	CLST << endl;

	CLST << "+ RECTIFIED BUILDINGS GEOMETRY +" << endl;
	CLST << "^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^" << endl;
	CLST << setw(COLUMN) << "Bldg IDs" << setw(COLUMN+8) << "P1(x,y) (m)" << setw(COLUMN+8) << "P2(x,y) (m)" << setw(COLUMN+8) << "P3(x,y) (m)" << setw(COLUMN+8) << "P4(x,y) (m)"  << setw(COLUMN) << "Height (m)" << endl;
	CLST << "--------------------------------------------------------------------------------------------------------------------------------------------------------" << endl;


	head();
	for(int i = 1; i <= getNoElms(); i++)
	{
		const Tetragon ttr = blg->getTetragon(Blg::SINGLE); const double hhh = blg->getHb();

		const int p1x = floor(ttr.pt(1).X() + 0.5);
		const int p1y = floor(ttr.pt(1).Y() + 0.5);
		const int p2x = floor(ttr.pt(2).X() + 0.5);
		const int p2y = floor(ttr.pt(2).Y() + 0.5);
		const int p3x = floor(ttr.pt(3).X() + 0.5);
		const int p3y = floor(ttr.pt(3).Y() + 0.5);
		const int p4x = floor(ttr.pt(4).X() + 0.5);
		const int p4y = floor(ttr.pt(4).Y() + 0.5);
		const double HHH = 0.1*floor(10*hhh + 0.5);
		CLST << setw(COLUMN) << i;
		CLST << setw(COLUMN) << p1x << setw(8) << p1y;
		CLST << setw(COLUMN) << p2x << setw(8) << p2y;
		CLST << setw(COLUMN) << p3x << setw(8) << p3y;
		CLST << setw(COLUMN) << p4x << setw(8) << p4y;
		CLST << setw(COLUMN) << HHH << endl;

		advance();
	}
	CLST << endl; head();
}

Building::~Building()
{

	ML_TRACE("Building::~Building");

	if(getNoElms() != 0)
	{
		head();
		delete [] blg;
		setNoElms(0); setPosition(0);
	}
}

void Building::redim(int n)
{
	ML_TRACE("Building::redim");

	if(getNoElms() != 0)
	{
		head();
		delete [] blg;
		setNoElms(0); setPosition(0);
	}

	blg = new Blg [n+1];
	setNoElms(n); setPosition(1);
}

void Building::advance()
{
	SimpleIterator::advance();
	blg++;
}

void Building::reverse()
{
	SimpleIterator::reverse();
	blg--;
}

void Building::prep()
{

	ML_TRACE("Building::normalize");

	createClusters();
	rectify();
}

double Building::distance(Tetragon ttr) const
{
	ML_TRACE("Building::distance(Tetragon ttr)");

	Rectangle3d rect1, rect2; Vector2d pt1,pt2,pt4,dpt;

	double f11,f12, f21,f22;
	double g11,g12, g21,g22;

	const Tetragon Ttr = getTetragon(SINGLE);

	pt1 = Ttr.pt(1);
	pt2 = Ttr.pt(2);
	pt4 = Ttr.pt(4);
	rect2.Origin() = Vector3d(pt1.X(), pt1.Y(), 0.0);
	dpt = pt2 - pt1;
	rect2.Edge0 () = Vector3d(dpt.X(), dpt.Y(), 0.0);
	dpt = pt4 - pt1;
	rect2.Edge1 () = Vector3d(dpt.X(), dpt.Y(), 0.0);

	pt1 = ttr.pt(1);
	pt2 = ttr.pt(2);
	pt4 = ttr.pt(4);
	rect1.Origin() = Vector3d(pt1.X(), pt1.Y(), 0.0);
	dpt = pt2 - pt1;
	rect1.Edge0 () = Vector3d(dpt.X(), dpt.Y(), 0.0);
	dpt = pt4 - pt1;
	rect1.Edge1 () = Vector3d(dpt.X(), dpt.Y(), 0.0);
	const double dist1 = Distance(rect1, rect2, &f11,&f12,&f21,&f22);


	// For checking accuracy.
	pt1 = ttr.pt(3);
	pt2 = ttr.pt(4);
	pt4 = ttr.pt(2);
	rect1.Origin() = Vector3d(pt1.X(), pt1.Y(), 0.0);
	dpt = pt2 - pt1;
	rect1.Edge0 () = Vector3d(dpt.X(), dpt.Y(), 0.0);
	dpt = pt4 - pt1;
	rect1.Edge1 () = Vector3d(dpt.X(), dpt.Y(), 0.0);
	const double dist2 = Distance(rect1, rect2, &g11,&g12,&g21,&g22);

	if(fabs(dist1 - dist2) > 0.01)
	{
		CLOG << "Building::distance(Tetragon ttr) : " << dist1 << ',' << dist2 << endl;
	}

	return min(dist1, dist2);
}

double Building::distance(Vector2d vec) const
{
	ML_TRACE("Building::distance(Vector2d vec)");

	Rectangle3d rect2; Vector2d pt1,pt2,pt4,dpt;

	const Tetragon Ttr = getTetragon(SINGLE);

	pt1 = Ttr.pt(1);
	pt2 = Ttr.pt(2);
	pt4 = Ttr.pt(4);

	rect2.Origin() = Vector3d(pt1.X(), pt1.Y(), 0.0);
	dpt = pt2 - pt1;
	rect2.Edge0 () = Vector3d(dpt.X(), dpt.Y(), 0.0);
	dpt = pt4 - pt1;
	rect2.Edge1 () = Vector3d(dpt.X(), dpt.Y(), 0.0);

	return Distance(Vector3d(vec.X(), vec.Y(), 0.0), rect2);
}

double Building::repScale(Blg blg_) const
{
	ML_TRACE("Building::repScale");

	const Tetragon ttr = blg_.getTetragon(Blg::SINGLE);
	return sqrt((ttr.pt(2) - ttr.pt(1)).Length()*(ttr.pt(4) - ttr.pt(1)).Length());
}

bool Building::proximity(Blg fix) const
{
	ML_TRACE("Building::proximity");

	const double  rep1 = repScale(*blg);
	const double  rep2 = repScale( fix);
	const Tetragon ttr = fix.getTetragon(Blg::SINGLE);

	// CLST << setw(COLUMN) << distance(ttr) << setw(COLUMN) << (rep1+rep2)*0.51 << endl;

	return (distance(ttr) < (rep1+rep2)*0.51);			// Experimental implementation.
}

void Building::createClusters()
{
	ML_TRACE("Building::createClusters");

	const int nr = getNoElms(); int* list  = new int [nr*nr+1];

	CLST << endl;
	CLST << endl;
	CLST << "<< Evaluated buildings proximity >>" << endl;
	CLST << endl;
	CLST << "+ PROXIMITY MATRIX (1=CLOSE,0=SEPARATE) +" << endl;
	CLST << "^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^" << endl;

	int i,j;

    for(i = 1; i <= nr; i++)			// Evaluate buildings proximity.
	{
	    head(); for(j = 1; j < i; j++) advance(); const Blg fix = *blg;

	    head();
	    for(j = 1; j <= nr; j++)
		{
		    list[j+(i-1)*nr] = proximity(fix);
		    CLST << setw(1) << list[j+(i-1)*nr];
		    advance();
		}

		CLST << endl;
	}

	CLST << endl;


	int* min_idx = new int [nr+1];
	int* prev    = new int [nr+1];
	for(i = 1; i <= nr; i++) min_idx[i] = prev[i] = i;
	while(1)
	{
		for(i = nr; i >= 1; i--)
		{
		    for(j = nr; j >= 1; j--)
			{
				if(list[j+(i-1)*nr])
				{
					// CLST << i << ',' << j << ',' << prev[j] << endl;
					min_idx[i] = min(min_idx[i], prev[j]);
				}
			}
		}


		bool converged = true;
		for(i = 1; i <= nr; i++)
		{
			if(prev[i] != min_idx[i]) converged = false;
			prev[i] = min_idx[i];
		}

		if(converged) break;
	}

	head();
    for(i = 1; i <= nr; i++)			// Set cluster IDs.
	{
	    setClusterIndex(min_idx[i]);
	    advance();
	}

	delete [] prev;
	delete [] min_idx;
	delete [] list;
}

void Building::rectify()
{
	ML_TRACE("Building::rectify");

	for(int i = 1; i <= getNoElms(); i++)
	{
		int cls, j;

		head();
		for(j = 1; j<=i; j++)
		{
			cls = getClusterIndex();
			advance();
		}

		if(cls > 0)
		{
			int pcount = 0;

			head();
			for(j = 1; j<=getNoElms(); j++)								// Check the number of verteces ...
			{
				if(cls == getClusterIndex()) pcount += 4;
				advance();
			}

			int p_index = 0; Vector2d* pts = new Vector2d[pcount];		// Assign vertex values ...

			head();
			for(j = 1; j<=getNoElms(); j++)
			{
				if(cls == getClusterIndex())
				{
					for(int k = 0; k < 4; k++)
					{
						// cout << "p_index=" << p_index << endl;
						pts[p_index] = getTetragon(SINGLE).pt(k+1); p_index++;
						// cout << "check=" << getTetragon(SINGLE).pt(k+1).X() << ',' << getTetragon(SINGLE).pt(k+1).Y() << endl;
					}
				}
				advance();
			}

			/*
			for(int iter=0; iter < pcount; iter++)
			{
				CLOG << pts[iter].X() << ',' << pts[iter].Y() << ',';
			}
			CLOG << endl;
			*/

			// Create convex hull.
			// Note. DEVIDE_AND_CONQUER method is rather erroneous. So we will use INCREMENTAL
			// method in 2.02 or later version.
			MyConvexHull* hll = new MyConvexHull(pts, pcount, MyConvexHull::INCREMENTAL);
			Rectify* rct = new Rectify2(hll); // cout << pcount << ',' << cls << endl;

			rct->exec();												// Rectify the convex hull ...
			Tetragon ttr = rct->getTetragon();

			delete rct;
			delete hll;
			delete [] pts;

			head();
			for(j = 1; j <= getNoElms(); j++)
			{
				if(cls == getClusterIndex())
				{
					setTetragon(CLUSTER, ttr);
					/*
					CLST << setw(COLUMN) << cls << setw(COLUMN) << ttr.pt(1).X() << setw(COLUMN) << ttr.pt(2).X() << setw(COLUMN) << ttr.pt(3).X() << setw(COLUMN) << ttr.pt(4).X() << endl;
					CLST << setw(COLUMN) << cls << setw(COLUMN) << ttr.pt(1).Y() << setw(COLUMN) << ttr.pt(2).Y() << setw(COLUMN) << ttr.pt(3).Y() << setw(COLUMN) << ttr.pt(4).Y() << endl;
					*/
					setClusterIndex(-cls);								// Attach sign flag for avoiding duplicate evaluation ...
				}
				advance();
			}
		}
	}

	CLST << endl;
	CLST << endl;
	CLST << "<< Evaluated groups w.r.t. proximity >>" << endl;
	CLST << endl;
	CLST << "+ INDIVIDUAL BUILDING'S GROUP IDs +" << endl;
	CLST << "^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^" << endl;
	CLST << setw(COLUMN) << "Bldg IDs" << setw(COLUMN) << "Group IDs" << setw((COLUMN+8)*4) << "Approximated rectangluar cluster verteces coordinates"<< endl;
	CLST << "--------------------------------------------------------------------------------------------------------------------------------------------------------" << endl;

	head();
	for(i = 1; i <= getNoElms(); i++)
	{
		int idx = getClusterIndex(); idx = (idx < 0 ? -idx : idx); setClusterIndex(idx);
		CLST << setw(COLUMN) << i << setw(COLUMN) << idx;

		const Tetragon ttr = getTetragon(Building::CLUSTER);
		CLST << setw(COLUMN) << floor(ttr.pt(1).X() + 0.5) << setw(8) << floor(ttr.pt(1).Y() + 0.5);
		CLST << setw(COLUMN) << floor(ttr.pt(2).X() + 0.5) << setw(8) << floor(ttr.pt(2).Y() + 0.5);
		CLST << setw(COLUMN) << floor(ttr.pt(3).X() + 0.5) << setw(8) << floor(ttr.pt(3).Y() + 0.5);
		CLST << setw(COLUMN) << floor(ttr.pt(4).X() + 0.5) << setw(8) << floor(ttr.pt(4).Y() + 0.5);
		CLST << endl;

		advance();
	}
	CLST << endl; head();
}
