#ifndef BUILDING_H
#define BUILDING_H

#include "blg.h"
#include "MyConvexHull.h"
#include "rectify2.h"
#include "../share/SimpleIterator.h"
#include "../share/FileFactory.h"

class Building : public SimpleIterator, public FileManager
{
    Blg* blg;
protected:
	virtual double repScale(Blg blg_) const;
    virtual bool   proximity(Blg fix) const;
    virtual void   createClusters();
    virtual void   rectify();
public:
    Building()
    {
		setIsFile(FileList::fnBuilding(FileList::IS, 1));
		setOsFile(FileList::fnBuilding(FileList::OS, 1));
	}
    virtual void redim(int n);
    virtual void scan ();
    virtual void print(){}
    virtual void lists();
	virtual void advance();
	virtual void reverse();

    virtual double distance (Tetragon ttr) const;	// Evaluate distance between rectangles.
    virtual double distance (Vector2d vec) const;	// Evaluate distance between point & rectangle.
    virtual void   prep();

    enum SingleOrCluster {SINGLE, CLUSTER};
    virtual void setTetragon(SingleOrCluster soc, Tetragon tetra)
    {
		soc == SINGLE ? blg->setTetragon(Blg::SINGLE, tetra) : blg->setTetragon(Blg::CLUSTER, tetra);
	}
    virtual Tetragon getTetragon(SingleOrCluster soc) const
    {
		return soc == SINGLE ? blg->getTetragon(Blg::SINGLE) : blg->getTetragon(Blg::CLUSTER);
	}

    virtual void     setClusterIndex(int idx) {blg->setClusterIndex(idx);}
	virtual int      getClusterIndex() const  {return blg->getClusterIndex();}

    virtual void   setHb(double hb) {blg->setHb(hb);}
    virtual double getHb() const {return blg->getHb();}

    virtual ~Building();
};

#endif
