#include "rectify2.h"

void Rectify2::exec()
{
	ML_TRACE("Rectify2::exec");

	const int no_v = hll->getNoVerteces();
	Vector2d* ver = new Vector2d [no_v+1];

	bool ok = false;
	double limit_deg = 5.0;
	double limit_cos = cos(M_PI/180.*limit_deg);

	int counter;
	while(limit_deg > 1.0)
	{
		counter = 0;
		for(int i = 1; i <= no_v; i++)
		{
			const int i_0 = i;
			const int i_m = (i == 1    ? no_v : i-1);
			const int i_p = (i == no_v ? 1    : i+1);

			const Vector2d p_0 = hll->pt(i_0);
			const Vector2d p_m = hll->pt(i_m);
			const Vector2d p_p = hll->pt(i_p);

			Vector2d dir1 = p_0 - p_m; dir1 = dir1/dir1.Length();
			Vector2d dir2 = p_p - p_0; dir2 = dir2/dir2.Length();

			if(dir1.Dot(dir2) < limit_cos)
			{
				counter++;
				ver[counter] = p_0;
			}
		}

		if(counter >= 4)
		{
			ok = true;
			break;
		}

		limit_deg = limit_deg - 0.5;
		limit_cos = cos(M_PI/180.*limit_deg);

		CLOG << "Rectify2::exec: Reducing the angle lower limit: " << setw(4) << limit_deg << endl;
	}

	if(!ok)
	{
		CERR << "Rectify2::exec: Failed in choosing convex hull verteces for rectification." << endl;
		exit(1);
	}

	const int NO_V = counter;

	double ctr_x = 0.0;
	double ctr_y = 0.0;
	double wgt = 0.0;
	for(int i = 1; i <= NO_V; i++)
	{
		const int i_0 = i;
		const int i_p = (i == NO_V ? 1    : i+1);

		const double dx = ver[i_p].X() - ver[i_0].X();
		const double dy = ver[i_p].Y() - ver[i_0].Y();
		const double dax = dx*dy/2.0;
		const double sax = dx*ver[i_0].Y();
		const double say = dy*ver[i_0].X();
		const double dcgx = ver[i_p].X() - dx/3.0;
		const double dcgy = ver[i_p].Y() - dy/3.0;
		const double scgx = ver[i_p].X() - dx/2.0;
		const double scgy = ver[i_p].Y() - dy/2.0;
		ctr_x += dax*dcgx + sax*scgx;
		ctr_y += dax*dcgy + say*scgy;
		wgt += dax + sax;
	}

	const Vector2d ctr = Vector2d(ctr_x/wgt, -ctr_y/wgt);

	bool* removed = new bool [NO_V*NO_V+1];
	for(int iter = 1; iter <= NO_V*NO_V; iter++) removed[iter] = false;

	const int NO_PAIRS = 5;
	int no_pairs = NO_PAIRS;
	int dia_1[NO_PAIRS + 1];
	int dia_2[NO_PAIRS + 1];
	int axis_1 = 0;
	int axis_2 = 0;
	for(iter = 1; iter <= NO_PAIRS; iter++)
	{
		bool find = false;
		int mx1;
		int mx2;
		double mxd = 0.0;
		for(int i = 1; i <= NO_V; i++)
		{
			for(int j = i+1; j <= NO_V; j++)
			{
				const bool exception = (i == axis_1 || i == axis_2 || j == axis_1 || j == axis_2);
				const int  idx = j + (i - 1)*NO_V;

				if(!removed[idx] && !exception)
				{
					const double dist = (ver[i]-ver[j]).Length();

					if(dist > mxd)
					{
						mxd = dist;
						mx1 = i;
						mx2 = j;
					}

					find = true;
				}
			}
		}

		if(!find)
		{
			no_pairs = iter - 1;
			break;
		}

		const int idx = mx2 + (mx1 - 1)*NO_V;
		removed[idx] = true;
		dia_1 [iter] = mx1;
		dia_2 [iter] = mx2;

		if(iter == 1)
		{
			axis_1 = mx1;
			axis_2 = mx2;
		}
	}

	/*
	cout << no_pairs << endl;
	*/

	const int P1 = dia_1[1];
	const int P2 = dia_2[1];
	const Vector2d PP = ver[P2] - ver[P1];
	const Vector2d NN = PP/PP.Length();
	int Q1;
	int Q2;

	/*
	cout << 1 << ',' << ver[P1].X() << ',' << ver[P1].Y() << endl;
	cout << 1 << ',' << ver[P2].X() << ',' << ver[P2].Y() << endl;
	*/

	double min_cosine = 1.0;
	for(i = 2; i <= no_pairs; i++)
	{
		const int q1 = dia_1[i];
		const int q2 = dia_2[i];
		const Vector2d qq = ver[q2] - ver[q1];
		const Vector2d mm = qq/qq.Length();

		const double cosine = fabs(NN.Dot(mm));

		/*
		cout << i << ',' << ver[q1].X() << ',' << ver[q1].Y() << endl;
		cout << i << ',' << ver[q2].X() << ',' << ver[q2].Y() << endl;
		cout << i << ',' << cosine << endl;;
		*/

		if(cosine < min_cosine)
		{
			min_cosine = cosine;
			Q1 = q1;
			Q2 = q2;
		}
	}

	/*

	cout << min_cosine << endl;
	*/

	const Vector2d QQ = ver[Q2] - ver[Q1];
	const Vector2d MM = QQ/QQ.Length();

	const double magnify = sqrt(hll->getConvexHullArea()/(fabs(NN.X()*MM.Y() - MM.X()*NN.Y())*2));

	ttr.pt(1) = ctr + magnify*NN;
	ttr.pt(2) = ctr + magnify*MM;
	ttr.pt(3) = ctr - magnify*NN;
	ttr.pt(4) = ctr - magnify*MM;

	delete [] removed;
	delete [] ver;
}
