#include "MetData2.h"
#include "stability2.h"
#include "statistics.h"

void Stability2::scan()
{
	met = new MetData2(); met->scan(); met->getGeoPosition(lon, lat, zzz);
	sun = new Sun2(lon, lat, met->getTimeZone());
	tim = new TimeCode(); tim->scan();

	met->lists();
	tim->lists();
}

void Stability2::run()
{
	// Create average daylight table
	const int nr = met->getNoDays();
	double* average = new double [nr];

	met->head(); int seq = 0; int prev_dy = 0; int cnt = 0; double sum = 0.0; int i;
	for(i = 1; i <= met->getNoElms(); i++)
	{
		// Placeholders for getting the meteorological data
		int _yr, _mn, _dy, _hr; double _wd, _ws, _tp, _dl, _sn;
		met->getMetData(_yr, _mn, _dy, _hr, _wd, _ws, _tp, _dl, _sn);

		if(i == 1)
		{
			prev_dy = _dy;
		}
		else if(_dy != prev_dy)
		{
			average[seq++] = (cnt != 0 ? sum/cnt : 9999); sum = cnt = 0; prev_dy = _dy;
			// cout << seq-1 << ',' << average[seq-1] << endl;
		}

		if((_hr >= 8 && _hr <= 17) && (_dl >= 0.0 && _dl <= 1.0))
		{
			sum += _dl;
			cnt++;
		}
		met->advance();
	}
	average[seq++] = (cnt != 0 ? sum/cnt : 9999); met->head();

	const string filename = getOsFile(); ofstream os(filename.c_str());


	CLST << endl;
	CLST << endl;
	CLST << "<< Evaluated stability parameters list >>" << endl;
	CLST << endl;
	CLST << "+ MODEL-READY METEOROLOGICAL DATA +" << endl;
	CLST << "^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^" << endl;
	CLST << setw(10) << "Day cls" << setw(10) << "Hour cls" << setw(10) << "Year" << setw(10) << "Month" << setw(10) << "Day" << setw(10) << "Hour" << setw(10) << "Wind dir" << setw(10) << "Wind spd" << setw(10) << "PG-cls" << setw(10) << "Temp." << setw(15) << "Sunrise-set" << endl;
	CLST << setw(10) << ""        << setw(10) << ""         << setw(10) << ""     << setw(10) << ""      << setw(10) << ""    << setw(10) << ""     << setw(10) << "(0-16)"   << setw(10) << "(m/s)"    << setw(10) << "(1-11)" << setw(10) << "(K)"   << setw(15) << "(hh:mm)"     << endl;
	CLST << "-------------------------------------------------------------------------------------------------------------------" << endl;

	Statistics* stat = new Statistics(tim->getMaxDailyCodes(), tim->getMaxHourlyCodes());

	met->head(); seq = 0; prev_dy = 0;
	for(i = 1; i <= met->getNoElms(); i++)
	{
		// Placeholders for getting the meteorological data
		int _yr, _mn, _dy, _hr; double _wd, _ws, _tp, _dl, _sn; double sr, ss;
		met->getMetData(_yr, _mn, _dy, _hr, _wd, _ws, _tp, _dl, _sn);

		if(i == 1)
		{
			prev_dy = _dy;
		}
		else if(_dy != prev_dy)
		{
			seq++; prev_dy = _dy;
		}
		sun->setDate(_yr, _mn, _dy); sun->setTime(_hr); const Sun::SunPosition sp = sun->getSunPosition(sr, ss);

		const int sr_hr = floor(sr/60);
		const int sr_mn = floor(sr - sr_hr*60);
		const int ss_hr = floor(ss/60);
		const int ss_mn = floor(ss - ss_hr*60);


		// Evaluate stalilities (First step)
		int res = table(sp, _dl, _ws, average[seq], _sn);

		// Evaluate stalilities (Second step. Some modification done with corrected wind speed)
		if(res != 9999)
		{
			double pp;
			if(res <= 2)
				pp = 0.07;
			else if(res <= 4)
				pp = 0.07;
			else if(res <= 6)
				pp = 0.10;
			else if(res <= 8)
				pp = 0.15;
			else if(res == 9)
				pp = 0.35;
			else
				pp = 0.55;

			const double ws = _ws*pow(10.0/zzz, pp);
			res = table(sp, _dl, ws, average[seq], _sn);
		}

		tim->setTime(_yr, _mn, _dy, _hr);
		const int _Dy = tim-> getDailyCode();
		const int _Hr = tim->getHourlyCode();

		os << setw(COLUMN) << _Dy << ',';
		os << setw(COLUMN) << _Hr << ',';
		os << setw(COLUMN) << _yr << ',';
		os << setw(COLUMN) << _mn << ',';
		os << setw(COLUMN) << _dy << ',';
		os << setw(COLUMN) << _hr << ',';
		os << setw(COLUMN) << _wd << ',';
		os << setw(COLUMN) << _ws << ',';
		os << setw(COLUMN) << res << ',';
		os << setw(COLUMN) << _tp << ',';
		os << setw(COLUMN) << _sn;
		os << endl;

		// Append to list file
		CLST << setw(10) << _Dy;
		CLST << setw(10) << _Hr;
		CLST << setw(10) << _yr;
		CLST << setw(10) << _mn;
		CLST << setw(10) << _dy;
		CLST << setw(10) << _hr;
		CLST << setw(10) << _wd;
		CLST << setw(10) << _ws;
		CLST << setw(10) << res;
		CLST << setw(10) << _tp;
		// CLST << setw(20) << _sn;
		CLST << setw(4)  << sr_hr << ':' << setw(2) << sr_mn << "<->";
		CLST << setw(2)  << ss_hr << ':' << setw(2) << ss_mn;
		CLST << endl;

		stat->add(_Dy, _Hr, _wd, _ws, res);

		met->advance();
	}
	met->head(); os.close(); stat->dump(); delete stat; CLST << endl; delete [] average;
}
