#include "TimeCode.h"
#include "../meteorology/ErrorHandler.h"

void TimeCode::scan()
{
	const string filename = getIsFile(); ifstream is(filename.c_str());
	
	is.exceptions(ios_base::goodbit);
	try{
		is.exceptions(ios_base::badbit | ios_base::failbit);
		
		for(int i = 0; i < 4; i++)
		{
			is >> year[i];
			is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
			is >> month[i];
			is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
			is >> day[i];
		}
		
		for(int j = 0; j < 4; j++)
		{
			is >> hour[j];
			is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
			is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		}
		is.close();
	}
	catch(...)
	{
		cerr << ErrorTimeCode_scan(filename);
		CERR << ErrorTimeCode_scan(filename);
		exit(1);
	}
		
	for(int i = 0; i < 4; i++)
	{
		const int _yr = year [i];
		const int _mn = month[i];
		const int _dy = day  [i];
		if((_yr < 1900 || _yr > 2200) || (_mn < 1 || _mn > 12) || (_dy < 1 || _dy > 31))
		{
			jul[i] = 9999999;
		}
		else
		{
			kal->setDate(_yr, _mn, _dy);
			jul[i] = kal->julian();
			nr_dy_codes = i+1;
		}
	}
	
	for(int j = 0; j < 4; j++)
	{
		const int _hr = hour [j];
		if(_hr < 1 || _hr > 24)
		{
			hrs[j] = 9999999;
		}
		else
		{
			if(hour[j] >= hour[0])
			{
				hrs[j] = hour[j] - hour[0];
			}
			else
			{
				hrs[j] = hour[j] - hour[0] + 24;
			}
			nr_hr_codes = j+1;
		}
	}
}

void TimeCode::print()
{
#if SCAN_CHECK > 0
	const string filename = getOsFile();

	ofstream os(filename.c_str());
	
	int i;

	for(i = 0; i < 4; i++)
	{
		os << setw(COLUMN) << year [i] << ',';
		os << setw(COLUMN) << month[i] << ',';
		os << setw(COLUMN) << day  [i] << endl;
	}
	
	for(i = 0; i < 4; i++)
	{
		os << setw(COLUMN) << hour [i] << ',,';
		os << endl;
	}
	os.close();
#endif
}

void TimeCode::lists()
{
	int i;

	CLST << endl;
	CLST << endl;
	CLST << "<< Scanned time code list >>" << endl;
	CLST << endl;
	
	CLST << "+ USER DEFINED DAY CLASSIFICATIONS +" << endl;
	CLST << "^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^" << endl;
	CLST << setw(40) << "Start or End dates (year/month/day)" << endl;
	CLST << "----------------------------------------" << endl;
	for(i = 0; i < 4; i++)
	{
		CLST << setw(10) << i + 1    << " : ";
		CLST << setw( 5) << year [i] << " / ";
		CLST << setw( 5) << month[i] << " / ";
		CLST << setw( 5) << day  [i] << endl;
	}
	CLST << endl;
	
	CLST << "+ USER DEFINED HOUR CLASSIFICATIONS +" << endl;
	CLST << "^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^" << endl;
	CLST << setw(20) << "Start or End hours" << endl;
	CLST << "--------------------" << endl;
	for(i = 0; i < 4; i++)
	{
		CLST << setw(10) << i + 1    << " : ";
		CLST << setw( 5) << hour [i] << endl;
	}
	CLST << endl;
}

int TimeCode::getDailyCode() const
{
	kal->setDate(yr, mn, dy); const int cur = kal->julian();
	for(int i = 3; i >= 0; i--)
	{
		if(cur >= jul[i])
			return i + 1;
	}
	cerr << "TimeCode::getDailyCode. Failed in returning the daily code." << endl;
	CERR << "TimeCode::getDailyCode. Failed in returning the daily code." << endl;
	exit(1);
}

int TimeCode::getHourlyCode () const
{
	int cur;
	
	if(hr >= hour[0])
	{
		cur = hr - hour[0];
	}
	else
	{
		cur = hr - hour[0] + 24;
	}
	
	for(int i = 3; i >= 0; i--)
	{
		if(cur >= hrs[i])
			return i + 1;
	}
	
	cerr << "TimeCode::getHourlyCode. Failed in returning the hourly code." << endl;
	CERR << "TimeCode::getHourlyCode. Failed in returning the hourly code." << endl;
	exit(1);
}
