#include "LineSrc.h"
#include "ErrorHandler.h"
#include "../share/common.h"

double LineSrc::getLineSrc(Link& lnk_, double& emi_) const
{
	ML_TRACE("LineSrc::getLineSrc");

	int      ide;
	Vector3d beg;
	Vector3d end;
	double   wid;
	string   des;

	lnk->getLink(ide, beg, end, wid, des);
	lnk_.setLink(ide, beg, end, wid, des);
	emi_ = *emi;

	return getOperationPercent();
}

LineSrc::~LineSrc()
{
	ML_TRACE("LineSrc::~LineSrc");

	if(getNoElms() > 0)
	{
		head();
		delete [] lnk;
		delete [] emi;
		setNoElms(0); setPosition(0);
	}
}

void LineSrc::redim(int n)
{
	ML_TRACE("LineSrc::redim");

	if(getNoElms() != 0)
	{
		head();
		delete [] lnk;
		delete [] emi;
		setNoElms(0); setPosition(0);
	}

	lnk = new Link   [n+1];
	emi = new double [n+1];
	setNoElms(n); setPosition(1);
}

void LineSrc::advance()
{
	SimpleIterator::advance();
	lnk++;
	emi++;
}

void LineSrc::reverse()
{
	SimpleIterator::reverse();
	lnk--;
	emi--;
}

void LineSrc::scan()
{
	ML_TRACE("LineSrc::scan");

	const string filename = getIsFile();
	ifstream  is(filename.c_str());

	is.exceptions(ios_base::goodbit);
	try{
		is.exceptions(ios_base::badbit | ios_base::failbit);

		int n; is >> n; redim(n); is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		double mol; is >> mol; setMolecularWeight(mol); is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');

		head();
		for(int i = 1; i <= getNoElms(); i++)
		{
			lnk->scan(is);
			is >> *emi;
			is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
			is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
			is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
			is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
			is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
			is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
			advance();
		}
		head();
		is.close();
	}
	catch(...)
	{
		cerr << ErrorLineSrc_scan(filename);
		CERR << ErrorLineSrc_scan(filename);
		exit(1);
	}
}

void LineSrc::print()
{
#if SCAN_CHECK > 0
	const string filename = getOsFile();
	ofstream  os(filename.c_str());

	os << "<< Scanned link coordinates & emission (ide,x1,y1,z1,x2,y2,z2,des,emi). " << getNoElms() << " data readed. >>" << ",," << endl;
	os << endl;

	head();
	for(int i = 1; i <= getNoElms(); i++)
	{
		lnk->print(os); os << ',' << setw(COLUMN) << *emi << endl;
		advance();
	}
	head();
	os.close();
#endif
}

void LineSrc::lists()
{
	ML_TRACE("LineSrc::lists");

	CLST << endl;
	CLST << endl;
	CLST << "<< Line sources data list >>" << endl;
	CLST << endl;
	CLST << "+ LINE SOURCES +" << endl;
	CLST << "^^^^^^^^^^^^^^^^" << endl;
	CLST << setw(COLUMN) << "Link IDs" << setw(COLUMN+8) << "P1(x,y) (m)" << setw(COLUMN+8) << "P2(x,y) (m)" << setw(COLUMN) << "Width (m)" << setw(COLUMN) << "Description" << setw(COLUMN) << "Emission (kg/m/s)" << endl;
	CLST << "----------------------------------------------------------------------------------------------------------------------------------------" << endl;
	head();
	for(int i = 1; i <= getNoElms(); i++)
	{
		lnk->lists(); CLST << setw(COLUMN) << *emi << endl;
		advance();
	}
	head(); CLST << endl;
}

