#include "MetiLisPointDropletSrcSimulator.h"
#include "MetiLisPasquillGifford.h"
#include "../share/WildMagic.h"

void MetiLisPointDropletSrcSimulator::scan()
{
	ML_TRACE("MetiLisPointDropletSrcSimulator::scan");

	droplet->scan();

	isTimeSeries() ? attachMetData(new MetTimeSeries()) : attachMetData(new Met());
	attachRcpData(new Rcp());
	attachSrcData(new PointDropletSrc(droplet));
	attachSgmData(new MetiLisPasquillGifford());
	attachPatData(new Pat());
	attachRanData(new Ran());
	attachBlgData(new Blg());

	ptr_blg->scan(); Simulator::scan(); if(ptr_rcp->getTerrainOpt()) attachAdjData(new TerrainAdjustment());
}

double MetiLisPointDropletSrcSimulator::wsMltfactor() const
{
	ML_TRACE("MetiLisPointDropletSrcSimulator::wsMltfactor");

	const double mlt = PointSrcSimulator::wsMltfactor ();

	// Placeholders for retrieving the building parameters
	double _w1_, _w2_, _hh_, _he_, _dx_, _dy_, _z1_, _z2_, _y1_, _y2_, _al_; int _tp_, _ef_;
	ptr_blg->getBlg(_w1_, _w2_, _hh_, _tp_, _he_, _dx_, _dy_, _z1_, _z2_, _y1_, _y2_, _al_, _ef_);

	if(TRACE_Q)
	{
		CLOG << "mlt =" << mlt  << endl;
		CLOG << "_al_=" << _al_ << endl;
	}

	if(_ef_ <= 0)
	{
		CERR << "Error detected in MetiLisPointDropletSrcSimulator::wsMltfactor." << endl;
		exit(1);
	}

	return mlt*_al_;
}

int MetiLisPointDropletSrcSimulator::assembleSimulationProperties(bool new_met, bool new_src)
{
	ML_TRACE("MetiLisPointDropletSrcSimulator::assembleSimulationProperties");

	if(new_met)
	{
		double _wd_, _ws_, _tp_; int _st_;
		ptr_met->getMet(_wd_, _ws_, _st_, _tp_);

		if(_wd_ < 0.0 || _ws_ <= 0.4 || fabs(_ws_ - 0.4) < 1.0e-5)
		{
			// Under the calm condition, setting to dummy axis index.
			setAxis(0);
			setCalmFlag(true);
		}
		else
		{
			// For selecting appropriate directional building parameters.
			const int pwa = ((int) (7 + (_wd_ + 11.25)/22.5)) % 16;
			if(pwa < 0 || pwa > 15)
			{
				CERR << "MetiLisPointDropletSrcSimulator::assembleSimulationProperties: Error!" << endl;
				exit(1);
			}

			setCalmFlag(false);
			setAxis(pwa);
		}

		const ListLevel level = getListLevel();
		if(level == MODERATE || level == FULL)
		{
			if(isCalm())
			{
				CLST << " WD=" << setw(2) << 0;
			}
			else
			{
				CLST << " WD=" << setw(2) << getAxis() + 1;
			}

			CLST << ' ';
		};

		setOriginal_wd(_wd_);
		setOriginal_ws(_ws_);
		setModelled_st(reclassST(_st_));
		ptr_sgm->setStability(getModelled_st());
	}
	if(new_src || new_met)
	{

		// Placeholders for source object
		Ring _rin_; bool _stk_, _plm_; double _emi_,_vel_,_gas_,_tpr_;
		const double percent = ptr_src->getPointSrc(_rin_, _stk_, _plm_, _emi_, _vel_, _gas_, _tpr_);
		setStk(_stk_);
		setPlm(_plm_);
		setEmi(_emi_*percent*0.01);
		setVel(_vel_*percent*0.01);
		setGas(_gas_*percent*0.01);
		setTpr(_tpr_);

		// Placeholders for ring object
		int  _ide_; Vector3d _ctr_; double _dia_,_hhh_; string _des_;
		_rin_.getRing(_ide_, _ctr_, _dia_, _hhh_, _des_);
		setIde(_ide_);
		setCtr(_ctr_);
		setDia(_dia_);
		setAlt(_hhh_);

		if((getListLevel() == Simulator::MODERATE || getListLevel() == Simulator::FULL))
		{
			CLST << "[" << setw(3) << _ide_ << ":" << setw(3) << floor(percent + 0.5) << "]";
		}

		// Evaluate building or plume rise effects.
		double ws_modified_with_power_law = getOriginal_ws()*PointSrcSimulator::wsMltfactor();
		ws_modified_with_power_law = max(1.0, ws_modified_with_power_law);
		evaluateDhAtStackPosition(ws_modified_with_power_law);
	}

	ptr_sgm->resetValidationFlags();

	if(getEmi() == 0.0) return 0;

	// Placeholder for local coordinates
	double x_loc; double y_loc;
	setModelled_wd(getOriginal_wd() + wdAddfactor());	// Considering randomized wind direction.
	getLocalCoor(x_loc, y_loc, getCtr().X() + getDX(), getCtr().Y() + getDY());

	setDownwindDistance(fabs(x_loc)); setCrosswindDistance(fabs(y_loc));

	if(isCalm())
	{
		return 2;
	}
	else if(getCrosswindDistance() > getDownwindDistance()*getTrigHalfOpenAngle(TAN) || x_loc > 0)
	{
		return 0;
	}
	else
	{
		return 1;
	}
}

void MetiLisPointDropletSrcSimulator::evaluateDhAtStackPosition(double ws_modified_with_power_law)
{
	ML_TRACE("MetiLisPointDropletSrcSimulator::evaluateDhAtStackPosition");

	const Vector3d vec = getCtr(); const double h_s = vec.Z();

	const int pwa = getAxis();
	ptr_blg->setPrincipalWindAxis(pwa);

	// Placeholders for retrieving the building parameters
	double _w1_, _w2_, _hh_, _he_, _dx_, _dy_, _z1_, _z2_, _y1_, _y2_, _al_; int _tp_, _ef_;
	const int id = ptr_blg->getBlg(_w1_, _w2_, _hh_, _tp_, _he_, _dx_, _dy_, _z1_, _z2_, _y1_, _y2_, _al_, _ef_);

	// CERR << setw(4) << id << '-' << setw(2) << pwa+1 << ',';

	const double gep = gepStackHeight(_hh_,_w2_);


	double pre;

	if(isCalm())
	{
		pre = briggs();
	}
	else
	{
		const double s = stacktip(ws_modified_with_power_law);
		const double c =  concawe(ws_modified_with_power_law);
		pre = s + c;
	}

	const double hef = h_s + pre;
	const ListLevel level = getListLevel();

	/*
	CLST << "evaluateDhAtStackPosition" << endl;
	CLST << _ef_ << endl;
	CLST << hef << endl;
	CLST << gep << endl;
	CLST << endl;
	*/

	double p_rise;

	if(_ef_ > 0 && hef <= gep && !isCalm())
	{
		p_rise = _he_ - h_s;

		// Using METI-LIS dispersion parameters...
		((MetiLisPasquillGifford*) ptr_sgm)->setParameters(_w1_,_w2_,_hh_,_z1_,_z2_,_y1_,_y2_);

		if(VALIDATION_Q)	// Normally VALIDATION_Q is set to false.
		{
			CLST << endl;
			CLST << endl;
			CLST << setw(30) << "pwa',alpha,hs=";
			CLST << setw(10) <<  pwa+1;
			CLST << setw(10) << _al_;
			CLST << setw(10) <<  h_s;
			CLST << endl;
			CLST << endl;

			setDX(0.00); setDY(0.00);
		}
		else
		{
			setDX(_dx_); setDY(_dy_);
		}

		const double ws_modified_with_metilis = getOriginal_ws()*wsMltfactor();
		setModelled_ws(max(1.0, ws_modified_with_metilis));

		if(level == MODERATE || level == FULL)
		{
			CLST << "{" << setw(5) << 0.1*floor(10*p_rise + 0.5) << "}";
		};
	}
	else
	{
		p_rise = pre;

		// Ignoring METI-LIS dispersion parameters...
		((MetiLisPasquillGifford*) ptr_sgm)->setParameters(0,0,0,0,0,0,0);

		if(VALIDATION_Q)	// Normally VALIDATION_Q is set to false.
		{
			CLST << endl;
			CLST << endl;
			CLST << setw(30) << "pwa',alpha,hs=";
			CLST << setw(10) <<  pwa+1;
			CLST << setw(10) <<  1;
			CLST << setw(10) <<  h_s;
			CLST << endl;
			CLST << endl;
		}

		setDX(0.00); setDY(0.00);
		setModelled_ws(isCalm() ? 0.0 : max(1.0, ws_modified_with_power_law));

		if(level == MODERATE || level == FULL)
		{
			CLST << "<" << setw(5) << 0.1*floor(10*p_rise + 0.5) << ">";
		}
	}

	setDhAtStackPosition(p_rise);
}
