#include "ErrorHandler.h"
#include "PointDropletSrc.h"

void PointDropletSrc::scan()
{
	ML_TRACE("PointDropletSrc::scan");

	const int nr_commas = max(6, droplet->getNoElms() - 1); int commas;

	const string filename = getIsFile();
	ifstream  is(filename.c_str());

	is.exceptions(ios_base::goodbit);
	try{
		is.exceptions(ios_base::badbit | ios_base::failbit);

		int    nnn; is >> nnn; redim(nnn);
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		double mol; is >> mol; setMolecularWeight(mol);
		for(commas = 1; commas < nr_commas; commas++)
			is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		commas = 0;

		head();
		for(int i = 1; i <= getNoElms(); i++)
		{
			rin->scan(is);

			for(commas = 6; commas < nr_commas; commas++)
				is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
			commas = 0;

			int accp;

			is >> accp; *stk = (accp == 0 ? false : true);
			is.ignore(NO_OF_CHARS_PER_ONE_LINE, ','); commas++;
			is >> accp; *plm = (accp == 0 ? false : true);
			is.ignore(NO_OF_CHARS_PER_ONE_LINE, ','); commas++;
			is >> *emi;
			is.ignore(NO_OF_CHARS_PER_ONE_LINE, ','); commas++;
			is >> *vel;
			is.ignore(NO_OF_CHARS_PER_ONE_LINE, ','); commas++;
			is >> *gas;
			is.ignore(NO_OF_CHARS_PER_ONE_LINE, ','); commas++;
			is >> *tpr;
			for(; commas < nr_commas; commas++)
				is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
			commas = 0;

			is >> *drg;
			for(; commas < nr_commas; commas++)
				is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
			commas = 0;

			for(int j = 1; j <= droplet->getNoElms(); j++)
			{
				double bd; is >> bd; bulk_density->setBulkDensity(j, bd);
				if(j != droplet->getNoElms())
				{
					is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
					commas++;
				}
			}
			for(; commas < nr_commas; commas++)
				is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
			commas = 0;

			for(int k = 1; k <= droplet->getNoElms(); k++)
			{
				double wr; is >> wr; weight_ratio->setWeightRatio(k, wr);
				if(k != droplet->getNoElms())
				{
					is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
					commas++;
				}
			}
			for(; commas < nr_commas; commas++)
				is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
			commas = 0;

			advance();
		}
		head();
		is.close();
	}
	catch(...)
	{
		cerr << ErrorPointDropletSrc_scan(filename);
		CERR << ErrorPointDropletSrc_scan(filename);
		exit(1);
	}
}

PointDropletSrc::~PointDropletSrc()
{
	ML_TRACE("PointDropletSrc::~PointDropletSrc");

	head();
	delete [] weight_ratio;
	delete [] bulk_density;
	delete [] drg;
}

void PointDropletSrc::redim(int n)
{
	ML_TRACE("PointDropletSrc::redim");

	if(getNoElms() != 0)
	{
		head();
		delete [] rin;
		delete [] stk;
		delete [] plm;
		delete [] emi;
		delete [] vel;
		delete [] gas;
		delete [] tpr;

		delete [] drg;

		delete [] bulk_density;
		delete [] weight_ratio;

		setNoElms(0); setPosition(0);
	}

	rin = new Ring   [n+1];
	stk = new bool   [n+1];
	plm = new bool   [n+1];
	emi = new double [n+1];
	vel = new double [n+1];
	gas = new double [n+1];
	tpr = new double [n+1];

	drg = new double [n+1];

	bulk_density = new BulkDensity [n+1];
	weight_ratio = new WeightRatio [n+1];

	setNoElms(n); setPosition(1);
}


void PointDropletSrc::advance()
{
	PointSrc::advance();
	drg++;
	bulk_density++;
	weight_ratio++;
}

void PointDropletSrc::reverse()
{
	PointSrc::reverse();
	drg--;
	bulk_density--;
	weight_ratio--;
}

void PointDropletSrc::print()
{
#if SCAN_CHECK > 0
	const string filename = getOsFile();
	ofstream  os(filename.c_str());

	os << "<< Scanned ring coordinates & point source properties (ide|x|y|dia|des|stk|plm|emi|vel|gas|tpr). " << getNoElms() << " data readed. >>" << ",," << endl;
	os << endl;

	head();
	for(int i = 1; i <= getNoElms(); i++)
	{
		rin->print(os); os << ',' << setw(COLUMN) << *stk << ',' << setw(COLUMN) << *plm << ',' << setw(COLUMN) << *emi << ',' << setw(COLUMN) << *vel << ',' << setw(COLUMN) << *gas << ',' << setw(COLUMN) << *tpr << endl;
		for(int j = 1; j <= droplet->getNoElms(); j++)
		{
			if(j < droplet->getNoElms())
			{
				os << setw(COLUMN);
				os << getWeightRatio(j) << ",";
			}
			else
			{
				os << setw(COLUMN);
				os << getWeightRatio(j) << endl;
			}
		}
		advance();
	}
	head();
	os.close();
#endif
}

void PointDropletSrc::lists()
{
	ML_TRACE("PointDropletSrc::lists");

	CLST << endl;
	CLST << endl;
	CLST << "<< Point source data list (Droplet data included) >>" << endl;
	CLST << endl;
	CLST << "+ INPUT POINT SOURCE DATA & OPTIONS +" << endl;
	CLST << "^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^" << endl;
	CLST << setw(COLUMN) << "Stack IDs" << setw(COLUMN) << "X (m)" << setw(COLUMN) << "Y (m)" << setw(COLUMN) << "Z (m)" << setw(COLUMN) << "Diameter (m)" << setw(COLUMN) << "H (m)" << setw(COLUMN) << "Description" << setw(COLUMN) << "StackTip(0-1)" << setw(COLUMN) << "Plume Rise (0-1)" << setw(COLUMN) << "Emission (kg/s)" << setw(COLUMN) << "Exit velocity (m/s)" << setw(COLUMN) << "Gas volume (m^3N/s)" << setw(COLUMN) << "Exit temp. (K)" << setw(COLUMN) << "Droplet infos ->";
	droplet->head();
	for(int k = 1; k <= droplet->getNoElms(); k++)
	{
		char buffer[50]; double val;
		val = 0.01*floor((droplet->getLower())*pow(10.0, 8) + 0.5); _gcvt(val, 8,  buffer); const string low = buffer;
		val = 0.01*floor((droplet->getUpper())*pow(10.0, 8) + 0.5); _gcvt(val, 8,  buffer); const string upp = buffer;
		CLST << setw(COLUMN) << low + " - " + upp;
		droplet->advance();
	}
	droplet->head(); CLST << endl;
	CLST << "--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------->" << endl;
	head();
	for(int i = 1; i <= getNoElms(); i++)
	{
		rin->lists(); CLST << setw(COLUMN) << *stk << setw(COLUMN) << *plm << setw(COLUMN) << *emi << setw(COLUMN) << *vel << setw(COLUMN) << *gas << setw(COLUMN) << *tpr << setw(COLUMN) << "<(%),(kg/m^3)>";
		for(int j = 1; j <= droplet->getNoElms(); j++)
		{
			CLST << setw(4) << "<" << setw(7) << 0.01*floor(10000*getWeightRatio(j) + 0.5) << "," << setw(7) << 0.001*floor(1000000*getBulkDensity(j) + 0.5) << ">";
		}
		CLST << endl; advance();
	}
	head(); CLST << endl;
}
