#include "PointSrc.h"
#include "ErrorHandler.h"
#include "../share/common.h"

double PointSrc::getPointSrc(Ring& rin_, bool& stk_, bool& plm_, double& emi_, double& vel_, double& gas_, double& tpr_) const
{
	ML_TRACE("PointSrc::getPointSrc");

	int      ide;
	Vector3d ctr;
	double   dia;
	double   hhh;
	string   des;

	rin->getRing(ide, ctr, dia, hhh, des);
	rin_.setRing(ide, ctr, dia, hhh, des);
	stk_ = *stk;
	plm_ = *plm;
	emi_ = *emi;
	vel_ = *vel;
	gas_ = *gas;
	tpr_ = *tpr;


	return getOperationPercent();
}

PointSrc::~PointSrc()
{
	ML_TRACE("PointSrc::~PointSrc");

	if(getNoElms() > 0)
	{
		head();
		delete [] rin;
		delete [] stk;
		delete [] plm;
		delete [] emi;
		delete [] vel;
		delete [] gas;
		delete [] tpr;
		setNoElms(0); setPosition(0);
	}
}

void PointSrc::redim(int n)
{
	ML_TRACE("PointSrc::redim");

	if(getNoElms() != 0)
	{
		head();
		delete [] rin;
		delete [] stk;
		delete [] plm;
		delete [] emi;
		delete [] vel;
		delete [] gas;
		delete [] tpr;
		setNoElms(0); setPosition(0);
	}

	rin = new Ring   [n+1];
	stk = new bool   [n+1];
	plm = new bool   [n+1];
	emi = new double [n+1];
	vel = new double [n+1];
	gas = new double [n+1];
	tpr = new double [n+1];
	setNoElms(n); setPosition(1);
}

void PointSrc::advance()
{
	SimpleIterator::advance();
	rin++;
	stk++;
	plm++;
	emi++;
	vel++;
	gas++;
	tpr++;
}

void PointSrc::reverse()
{
	SimpleIterator::reverse();
	rin--;
	stk--;
	plm--;
	emi--;
	vel--;
	gas--;
	tpr--;
}

void PointSrc::scan()
{
	ML_TRACE("PointSrc::scan");

	const string filename = getIsFile();
	ifstream  is(filename.c_str());

	is.exceptions(ios_base::goodbit);
	try{
		is.exceptions(ios_base::badbit | ios_base::failbit);

		int n; is >> n; redim(n);
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		double mol; is >> mol; setMolecularWeight(mol);
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');

		head();
		for(int i = 1; i <= getNoElms(); i++)
		{
			rin->scan(is);
			is >> *stk;
			is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
			is >> *plm;
			is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
			is >> *emi;
			is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
			is >> *vel;
			is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
			is >> *gas;
			is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
			is >> *tpr;
			advance();
		}
		head();
		is.close();
	}
	catch(...)
	{
		cerr << ErrorPointSrc_scan(filename);
		CERR << ErrorPointSrc_scan(filename);
		exit(1);
	}
}

void PointSrc::print()
{
#if SCAN_CHECK > 0
	const string filename = getOsFile();
	ofstream  os(filename.c_str());

	os << "<< Scanned ring coordinates & point source properties (ide|x|y|z|dia|hhh|des|stk|plm|emi|vel|gas|tpr). " << getNoElms() << " data readed. >>" << ",," << endl;
	os << endl;

	head();
	for(int i = 1; i <= getNoElms(); i++)
	{
		rin->print(os); os << ',' << setw(COLUMN) << *stk << ',' << setw(COLUMN) << *plm << ',' << setw(COLUMN) << *emi << ',' << setw(COLUMN) << *vel << ',' << setw(COLUMN) << *gas << ',' << setw(COLUMN) << *tpr << endl;
		advance();
	}
	head();
	os.close();
#endif
}

void PointSrc::lists()
{
	ML_TRACE("PointSrc::lists");

	CLST << endl;
	CLST << endl;
	CLST << "<< Point source data list >>" << endl;
	CLST << endl;
	CLST << "+ INPUT POINT SOURCE DATA & OPTIONS +" << endl;
	CLST << "^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^" << endl;
	CLST << setw(COLUMN) << "Stack IDs" << setw(COLUMN) << "X (m)" << setw(COLUMN) << "Y (m)" << setw(COLUMN) << "Z (m)" << setw(COLUMN) << "Diameter (m)" << setw(COLUMN) << "H (m)" << setw(COLUMN) << "Description" << setw(COLUMN) << "StackTip(0-1)" << setw(COLUMN) << "Plume Rise (0-1)" << setw(COLUMN) << "Emission (kg/s)" << setw(COLUMN) << "Exit velocity (m/s)" << setw(COLUMN) << "Gas volume (m^3N/s)" << setw(COLUMN) << "Exit temp. (K)" << endl;
	CLST << "--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------" << endl;
	head();
	for(int i = 1; i <= getNoElms(); i++)
	{
		rin->lists(); CLST << setw(COLUMN) << *stk << setw(COLUMN) << *plm << setw(COLUMN) << *emi << setw(COLUMN) << *vel << setw(COLUMN) << *gas << setw(COLUMN) << *tpr << endl;
		advance();
	}
	head(); CLST << endl;
}
