#include "TerrainAdjustment.h"
#include "ErrorHandler.h"

double TerrainAdjustment::simpl() const
{
	ML_TRACE("TerrainAdjustment::simpl");

	const double h_e = h_s + d_h;
	double  chop_off = level_at_recep_position - level_at_stack_position;

	if(h_s < chop_off)
	{
		//	... if the terrain height z - zs exceeds the source release height, hs ,
		//	the elevation of the receptor is automatically "chopped off" at
		//	the physical release height ... (ISC3 users manual)

		chop_off = h_s; const double e_x = chop_off - h_s;

		CLOG << ErrorTerrainAdjustment_simpl(i_d, x_x, y_y, e_x);
	}

	return (h_e - chop_off > 0.0 ? h_e - chop_off : 0.0);
}

double TerrainAdjustment::cmplx() const
{
	ML_TRACE("TerrainAdjustment::cmplx");

	const double h_e = h_s + d_h;
	const double h_t = level_at_recep_position - level_at_stack_position;
	double tmp = h_e - (1 - adjustmentFactor())*h_t;

	if(tmp < 10.0)
	{
		//	The plume centerline height is never less than 10 m
		//	above the ground level in complex terrain. (ISC3 users manual)
		tmp = 10.0;
	}

	return tmp;
}

double TerrainAdjustment::attenuation() const
{
	ML_TRACE("TerrainAdjustment::attenuation");

	const double dhr = cmplx() - z_z;

	if(false)           {/* dummy */}
	else if(!stb)       {return 1.0;}
	else if(dhr <   0.0){return 1.0;}
	else if(dhr > 400.0){return 0.0;}

	return (400 - dhr)/400;
}

TerrainAdjustment::TerrainStackRelation TerrainAdjustment::relation() const
{
	ML_TRACE("TerrainAdjustment::relation");

	const double h_e = h_s + d_h; const double zzz = level_at_recep_position - level_at_stack_position;

	if(zzz > h_s && zzz < h_e)
		return INTERMEDIATE;
	if(zzz > h_s)
		return COMPLEX;
	return SIMPLE;
}
