#ifndef TERRAIN_ADJUSTMENT_H
#define TERRAIN_ADJUSTMENT_H

//
// NOTES.
//
// 1. The current terrain adjustment software cannot simulate coupling problem
//    with the building downwash even if under the simple terrain.
// 2. The class is applicable in case you switch off building configuration.
// 3. Application will be restricted to the simple terrain scheme although some
//    functions has been implemented for evaluating the complex terrain.
//
//    More appropriate coupling and general problems following ISC3 will be
//    probably done in future version.
//

class TerrainAdjustment
{
	int    i_d;						// Stack id
	double x_x, y_y;				// Receptor coordinate
	double d_h;						// Net plume rise under the flat terrain condition (Minus sign acceptable)
	double h_s;						// Stack height measured from surface 
	double level_at_stack_position;
	double level_at_recep_position;
	bool   stb;						// Stable = true, Otherwise = false
	
	double mix;						// Reserved for future development (Mixing height)
	double z_z;						// Reserved for future development (Receptor height measured from surface)
protected:
	virtual double adjustmentFactor() const {return (stb ? 0.0 : 0.5);}
public:
	TerrainAdjustment(){}
	
	void setDebugInfos(int     i_d_, double  x_x_, double  y_y_)       {i_d  = i_d_; x_x  = x_x_; y_y  = y_y_;}
	void getDebugInfos(int&    i_d_, double& x_x_, double& y_y_) const {i_d_ = i_d ; x_x_ = x_x ; y_y_ = y_y ;}
	void setParameters(double  d_h_, double  h_s_, double  level_at_stack_position_, double  level_at_recep_position_)
	{
		d_h = d_h_;
		h_s = h_s_;
		level_at_stack_position = level_at_stack_position_;
		level_at_recep_position = level_at_recep_position_;
		stb = false;
		mix = 99999;
		z_z = 0;
	}
	void getParameters(double& d_h_, double& h_s_, double& level_at_stack_position_, double& level_at_recep_position_) const
	{
		d_h_ = d_h;
		h_s_ = h_s;
		level_at_stack_position_ = level_at_stack_position;
		level_at_recep_position_ = level_at_recep_position;
	}
	void setParameters(double  d_h_, double  h_s_, double  level_at_stack_position_, double  level_at_recep_position_, bool  stb_)
	{
		d_h = d_h_;
		h_s = h_s_;
		level_at_stack_position = level_at_stack_position_;
		level_at_recep_position = level_at_recep_position_;
		stb = stb_;
		mix = 99999;
		z_z = 0;
	}
	void getParameters(double& d_h_, double& h_s_, double& level_at_stack_position_, double& level_at_recep_position_, bool& stb_) const
	{
		d_h_ = d_h;
		h_s_ = h_s;
		level_at_stack_position_ = level_at_stack_position;
		level_at_recep_position_ = level_at_recep_position;
		stb_ = stb;
	}
	
	enum    TerrainStackRelation {SIMPLE, COMPLEX, INTERMEDIATE};
	virtual TerrainStackRelation relation() const;
	
	//	... simple terrain (i.e., terrain that exceeds the stack base elevation but is below the release height) ...
	virtual double simpl() const;
	virtual double cmplx() const;
	
	// In the current version the adjustment function is available on the effective stack height only.
	// The following adjustment functions are NOT implemented in this version because of lack in experiments.
	// virtual double sz() const;
	// virtual double sy() const;
	
	// The following function implemented but not used in the current version.
	virtual double attenuation() const;
	
	virtual ~TerrainAdjustment(){}
};

#endif
