#include "met.h"
#include "ErrorHandler.h"
#include "../share/common.h"

Met::~Met()
{
	ML_TRACE("Met::~Met");

	if(getNoElms() > 0)
	{
		head();
		delete [] wd;
		delete [] ws;
		delete [] st;
		delete [] tp;
		setNoElms(0); setPosition(0);
	}
}

void Met::setMet(double  wd_, double  ws_, int  st_, double  tp_)
{
	ML_TRACE("Met::setMet");

	*wd = wd_;
	*ws = ws_;
	*st = st_;
	*tp = tp_;
}

void Met::getMet(double& wd_, double& ws_, int& st_, double& tp_) const
{
	ML_TRACE("Met::getMet");

	wd_ = *wd;
	ws_ = *ws;
	st_ = *st;
	tp_ = *tp;
}

void Met::redim(int n)
{
	ML_TRACE("Met::redim");

	if(getNoElms() > 0)
	{
		head();
		delete [] wd;
		delete [] ws;
		delete [] st;
		delete [] tp;
		setNoElms(0); setPosition(0);
	}

	wd = new double [n+1];
	ws = new double [n+1];
	st = new int    [n+1];
	tp = new double [n+1];
	setNoElms(n); setPosition(1);
}

void Met::advance()
{
	SimpleIterator::advance();
	wd++;
	ws++;
	st++;
	tp++;
}

void Met::reverse()
{
	SimpleIterator::reverse();
	wd--;
	ws--;
	st--;
	tp--;
}

void Met::scan()
{
	ML_TRACE("Met::scan");

	const string filename = getIsFile();
	ifstream  is(filename.c_str());

	is.exceptions(ios_base::goodbit);
	try{
		is.exceptions(ios_base::badbit | ios_base::failbit);

		double dbl_placeholder;

		is >> dbl_placeholder; setAnemometerHeight(dbl_placeholder);
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is >> dbl_placeholder; setAverageSeconds  (dbl_placeholder);
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is >> dbl_placeholder; setPower4AverageSeconds(dbl_placeholder);
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');

		int n; is >> n; redim(n);
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');

		head();
		while(getPosition() <= getNoElms())
		{
			double _wd;
			double _ws;
			int    _st;
			double _tp;

			is >> _wd;
			is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
			is >> _ws;
			is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
			is >> _st;
			is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
			is >> _tp;

			*wd = wdConverter(_wd);
			*ws = _ws;
			*st = _st;
			*tp = _tp;
			advance();
		}
		head();
		is.close();
	}
	catch(...)
	{
		cerr << ErrorMet_scan(filename);
		CERR << ErrorMet_scan(filename);
		exit(1);
	}
}

void Met::print()
{
#if SCAN_CHECK > 0
	const string filename = getOsFile();
	ofstream  os(filename.c_str());

	os << "<< Scanned meteorological data (wd|ws|st|tp). " << getNoElms() << " data readed. >>" << "," << meters << ",," << endl;
	os << endl;

	head();
	while(getPosition() <= getNoElms())
	{
		os << setw(COLUMN) << wdConverter(*wd) << ',' << setw(COLUMN) << *ws << ',' << setw(COLUMN) << *st << ',' << setw(COLUMN) << *tp << endl;
		advance();
	}
	head();
	os.close();
#endif
}

void Met::lists()
{
	ML_TRACE("Met::lists");

	CLST << endl;
	CLST << endl;
	CLST << "<< Time Averaging Parameters >>" << endl;
	CLST << setw(COLUMN) << "Avg. Time (sec)"   << setw(COLUMN) << "P-value" << endl;
	CLST << "----------------------------------------" << endl;
	CLST << setw(COLUMN) << getAverageSeconds() << setw(COLUMN) << getPower4AverageSeconds() << endl;
	CLST << endl;

	CLST << endl;
	CLST << endl;
	CLST << "<< Meteorological data list >>" << endl;
	CLST << endl;
	CLST << "+ METEOROLOGICAL DATA (SHORT TERM) +" << endl;
	CLST << "^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^" << endl;
	CLST << setw(COLUMN) << "Case" << setw(COLUMN) << "Wind dir (deg)" << setw(COLUMN) << "Wind Speed (m/s)" << setw(COLUMN) << "Stability (1-7)" << setw(COLUMN) << "Temperature (K)" << endl;
	CLST << "----------------------------------------------------------------------------------------------------" << endl;
	head();
	for(int i = 1; i <= getNoElms(); i++)
	{
		CLST << setw(COLUMN) << i << setw(COLUMN) << wdConverter(*wd) << setw(COLUMN) << *ws << setw(COLUMN) << *st << setw(COLUMN) << *tp << endl;
		advance();
	}
	CLST << endl;
	CLST << "* Stability Code Table *" << endl;
	CLST << "^^^^^^^^^^^^^^^^^^^^^^^^" << endl;
	CLST << setw(10) << "Code" << setw(10) << "PG Symb" << setw(10) << "Used as" << endl;
	CLST << "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~" << endl;
	CLST << setw(10) << " 1" << setw(10) << " A" << setw(10) << " A" << endl;
	CLST << setw(10) << " 2" << setw(10) << " B" << setw(10) << " B" << endl;
	CLST << setw(10) << " 3" << setw(10) << " C" << setw(10) << " C" << endl;
	CLST << setw(10) << " 4" << setw(10) << "DD" << setw(10) << " D" << endl;
	CLST << setw(10) << " 5" << setw(10) << "DN" << setw(10) << " D" << endl;
	CLST << setw(10) << " 6" << setw(10) << " E" << setw(10) << " E" <<endl;
	CLST << setw(10) << " 7" << setw(10) << " F" << setw(10) << " F" << endl;
	CLST << "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~" << endl;
	CLST << endl;
	head(); CLST << endl;
}
