#include "ran.h"
#include "ErrorHandler.h"
#include "../share/common.h"

Ran::~Ran()
{
	ML_TRACE("Ran::~Ran");

	if(getNoElms() > 0)
	{
		head();
		delete [] ra;
		setNoElms(0); setPosition(0);
	}
}

void Ran::open()
{
	ML_TRACE("Ran::open");

	const string filename = getIsFile(); is.open(filename.c_str());

	is.exceptions(ios_base::goodbit);
	try{
		is.exceptions(ios_base::badbit | ios_base::failbit);

		int n; is >> n; redim(n);
		for(int i = 1; i < n; i++)
			is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
		head();
	}
	catch(...)
	{
		cerr << ErrorRan_open(filename).message();
		CERR << ErrorRan_open(filename).message();
		exit(1);
	}
}

void Ran::close()
{
	ML_TRACE("Ran::close");

	is.close();
}

void Ran::setRan(double  ra_)
{
	ML_TRACE("Ran::setRan");

	*ra = ra_;
}

void Ran::getRan(double& ra_) const
{
	ML_TRACE("Ran::getRan");

	ra_ = *ra;
}

void Ran::redim(int n)
{
	ML_TRACE("Ran::redim");

	if(getNoElms() > 0)
	{
		head();
		delete [] ra;
		setNoElms(0); setPosition(0);
	}

	ra = new double [n+1];
	setNoElms(n); setPosition(1);
}

void Ran::advance()
{
	SimpleIterator::advance();
	ra++;
}

void Ran::reverse()
{
	SimpleIterator::reverse();
	ra--;
}

void::Ran::scan()
{
	ML_TRACE("Ran::scan");

	is.exceptions(ios_base::goodbit);
	try{
		is.exceptions(ios_base::badbit | ios_base::failbit);

		head();
		while(getPosition() <= getNoElms())
		{
			double _ra;
			is >>  _ra; if(getPosition() != getNoElms()) is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
			*ra =  _ra; advance();
		}
		head();
	}
	catch(...)
	{
		const string filename = getIsFile();
		cerr << ErrorRan_scan(filename);
		CERR << ErrorRan_scan(filename);
		exit(1);
	}
}

void Ran::print()
{
#if SCAN_CHECK > 0
	const string filename = getOsFile();
	ofstream  os(filename.c_str());

	head();
	while(getPosition() <= getNoElms())
	{
		os << setw(COLUMN) << *ra << ',';
		advance();
	}
	head();
	os << endl; os.close();
#endif
}

void Ran::lists()
{
	ML_TRACE("Ran::lists");

	CLST << endl;
	CLST << endl;
	CLST << "<< Generated random values list (Software generates fresh random values at each project) >>" << endl;
	CLST << endl;
	CLST << "+ USING " << setw(3) << getNoElms() << " RANDOM NUMBERS (-11.25 <= r <= +11.25) +" << endl;
	CLST << "^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^" << endl;
	head();
	while(getPosition() <= getNoElms())
	{
		CLST << setw(COLUMN) << *ra; advance();
	}
	head(); CLST << endl;
}
