#include "rcp.h"
#include "ErrorHandler.h"
#include "../share/common.h"

Rcp::~Rcp()
{
  ML_TRACE("Rcp::~Rcp");

  if(getNoElms() > 0)
    {
      head();
      delete [] vec;
      delete [] x__;
      delete [] y__;
      delete [] z__;
      delete [] hhh;
      delete [] cmm;
      delete [] va1;
      delete [] va2;
      delete [] flg;
      setNoElms(0); setPosition(0);
    }
}

void Rcp::redim(int n)
{
  ML_TRACE("Rcp::redim");

  if(getNoElms() != 0)
    {
      head();
      delete [] vec;
      delete [] x__;
      delete [] y__;
      delete [] z__;
      delete [] hhh;
      delete [] cmm;
      delete [] va1;
      delete [] va2;
      delete [] flg;
      setNoElms(0); setPosition(0);
    }

  vec = new Vector3d [n+1];
  x__ = new string   [n+1];
  y__ = new string   [n+1];
  z__ = new string   [n+1];
  hhh = new double   [n+1];
  cmm = new string   [n+1];
  va1 = new double   [n+1];
  va2 = new double   [n+1];
  flg = new double   [n+1];
  setNoElms(n); setPosition(1);
}

void Rcp::init()
{
  ML_TRACE("Rcp::init");

  head();
  for(int i = 1; i<=getNoElms(); i++)
    {
      *va1 = 0.0;
      *va2 = 0.0;
      *flg = 1.0;
      advance();
    }
  head();
}

void Rcp::advance()
{
  SimpleIterator::advance();
  vec++;
  x__++;
  y__++;
  z__++;
  hhh++;
  cmm++;
  va1++;
  va2++;
  flg++;
}

void Rcp::reverse()
{
  SimpleIterator::reverse();
  vec--;
  x__--;
  y__--;
  z__--;
  hhh--;
  cmm--;
  va1--;
  va2--;
  flg--;
}

void Rcp::scan()
{
  ML_TRACE("Rcp::scan");

  const string filename = getIsFile();
  ifstream  is(filename.c_str());

  is.exceptions(ios_base::goodbit);
  try{
    is.exceptions(ios_base::badbit | ios_base::failbit);

    int n; is >> n; redim(n);
    is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
    is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
    is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
    is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');

    int b; is >> b; setTerrainOpt(b);
    is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
    is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
    is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');
    is.ignore(NO_OF_CHARS_PER_ONE_LINE, ',');

    is.ignore(NO_OF_CHARS_PER_ONE_LINE, '\n');

    head();
    for(int i = 1; i <= getNoElms(); i++)
      {
        string row;
        string h__;
        string s;
        int  pos;

        getline (is, row);

        pos  = row.find (',');
        *x__ = row.substr (0, pos);
        row  = row.substr (pos + 1);

        pos  = row.find (',');
        *y__ = row.substr (0, pos);
        row  = row.substr (pos + 1);

        pos  = row.find (',');
        *z__ = row.substr (0, pos);
        row  = row.substr (pos + 1);

        pos  = row.find (',');
        h__  = row.substr (0, pos);
        row  = row.substr (pos + 1);

        pos  = row.find ('\n');
        s    = row.substr (0, pos);

        vec->X() = atof ((*x__).c_str ());
        vec->Y() = atof ((*y__).c_str ());
        vec->Z() = atof ((*z__).c_str ());
        *hhh     = atof (  h__. c_str ());
        *cmm     = s;
        advance();
      }
    head();
    is.close();
  }
  catch(...)
    {
      cerr << ErrorRcp_scan(filename);
      CERR << ErrorRcp_scan(filename);
      exit(1);
    }
}

void Rcp::print(bool b_val)
{
  ML_TRACE("Rcp::print");

  const string filename = getOsFile();
  ofstream  os(filename.c_str());

  head();
  for(int i = 1; i <= getNoElms(); i++)
    {
      const string xx = *x__;
      const string yy = *y__;
      const string zz = *z__;
      const double hh = *hhh;
      const string ss = *cmm;
      const double cc = *va1;
      const double dd = *va2;
      const int    vv = (fabs(*flg) > 1e-6 ? 1 : 0);
      if(b_val)
        os << setw(COLUMN) << xx << ',' << setw(COLUMN) << yy << ',' << setw(COLUMN) << zz << ',' << setw(COLUMN) << hh << ',' << setw(COLUMN) << cc << ',' << setw(COLUMN) << dd  << ',' << setw(COLUMN) << vv << ',' << setw(COLUMN) << ss << endl;
      else
        os << setw(COLUMN) << xx << ',' << setw(COLUMN) << yy << ',' << setw(COLUMN) << zz << ',' << setw(COLUMN) << hh << ',' << setw(COLUMN) << ss << endl;
      advance();
    }
  head();
  os.close();
}

void Rcp::lists()
{
  ML_TRACE("Rcp::lists()");

  CLST << endl;
  CLST << endl;
  CLST << "<< Receptor coordinates >>" << endl;
  CLST << endl;
  CLST << "+ RECEPTOR THREE DIMENSIONAL COORDINATES & ALTITUDE +" << endl;
  CLST << "^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^" << endl;
  CLST << setw(COLUMN) << "X (m)" << setw(COLUMN) << "Y (m)" << setw(COLUMN) << "Z (m)" << setw(COLUMN) << "H (m)" << setw(COLUMN) << "DESCRIPTION" << endl;
  CLST << "----------------------------------------------------------------------------------------------------" << endl;
  head();
  for(int i = 1; i <= getNoElms(); i++)
    {
      const string xx = *x__;
      const string yy = *y__;
      const string zz = *z__;
      const double hh = *hhh;
      const string ss = *cmm;
      CLST << setw(COLUMN) << xx << setw(COLUMN) << yy << setw(COLUMN) << zz << setw(COLUMN) << hh << setw(COLUMN) << ss << endl;
      advance();
    }
  head(); CLST << endl;
}

void Rcp::lists(int cs_code, bool verbose)
{
  ML_TRACE("Rcp::lists(int cs_code, bool verbose)");

  CLST << endl;
  CLST << endl;
  CLST << endl;
  CLST << endl;
  CLST << "<< Evaluated concentrations at user defined receptor coordinates: Case " << setw(4) << cs_code << " >>" << endl;
  if(verbose)
    {
      CLST << endl;
      CLST << "+ RECEPTOR THREE DIMENSIONAL COORDINATES, ALTITUDES, CONCENTRATIONS, DEPOSITION FLUXES AND VALIDATION FLAGS +" << endl;
      CLST << "^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^" << endl;
      if(getConcentrationUnitFlag())
        CLST << setw(COLUMN) << "X (m)" << setw(COLUMN) << "Y (m)" << setw(COLUMN) << "Z (m)" << setw(COLUMN) << "H (m)" << setw(COLUMN) << "Conc. (m^3/m^3)" << setw(COLUMN) << "Flux (kg/m^2/s)" << setw(COLUMN) << "Validated (0-1)" << setw(COLUMN) << "DESCRIPTION" << endl;
      else
        CLST << setw(COLUMN) << "X (m)" << setw(COLUMN) << "Y (m)" << setw(COLUMN) << "Z (m)" << setw(COLUMN) << "H (m)" << setw(COLUMN) << "Conc. (kg/m^3N)" << setw(COLUMN) << "Flux (kg/m^2/s)" << setw(COLUMN) << "Validated (0-1)" << setw(COLUMN) << "DESCRIPTION" << endl;
      CLST << "----------------------------------------------------------------------------------------------------------------------------------------------------------------" << endl;
    }
  double maxc = -1.0; double maxc_x, maxc_y, maxc_z, maxc_h;
  double maxd = -1.0; double maxd_x, maxd_y, maxd_z, maxd_h;
  double maxC = -1.0; double maxC_x, maxC_y, maxC_z, maxC_h;
  double maxD = -1.0; double maxD_x, maxD_y, maxD_z, maxD_h;
  head();
  for(int i = 1; i <= getNoElms(); i++)
    {
      const string xx = *x__;
      const string yy = *y__;
      const string zz = *z__;
      const string ss = *cmm;
      const double hh = *hhh;
      const double cc = *va1;
      const double dd = *va2;
      const int    vv = (fabs(*flg) > 1e-6 ? 1 : 0);
      if(verbose) CLST << setw(COLUMN) << xx << setw(COLUMN) << yy << setw(COLUMN) << zz << setw(COLUMN) << hh << setw(COLUMN) << cc << setw(COLUMN) << dd  << setw(COLUMN) << vv << setw(COLUMN) << ss << endl;
      if(vv == 1 && cc > maxc)
        {
          maxc = cc;
          maxc_x = atof (xx.c_str ());
          maxc_y = atof (yy.c_str ());
          maxc_z = atof (zz.c_str ());
          maxc_h = hh;
        }
      if(vv == 1 && dd > maxd)
        {
          maxd = dd;
          maxc_x = atof (xx.c_str ());
          maxc_y = atof (yy.c_str ());
          maxc_z = atof (zz.c_str ());
          maxd_h = hh;
        }
      if(cc > maxC)
        {
          maxC = cc;
          maxc_x = atof (xx.c_str ());
          maxc_y = atof (yy.c_str ());
          maxc_z = atof (zz.c_str ());
          maxC_h = hh;
        }
      if(dd > maxD)
        {
          maxD = dd;
          maxc_x = atof (xx.c_str ());
          maxc_y = atof (yy.c_str ());
          maxc_z = atof (zz.c_str ());
          maxD_h = hh;
        }
      advance();
    }
  if(maxc == 0.0){maxc_x = maxc_y = maxc_z = maxc_h = -99999;}
  if(maxC == 0.0){maxC_x = maxC_y = maxC_z = maxC_h = -99999;}
  if(maxd == 0.0){maxd_x = maxd_y = maxd_z = maxd_h = -99999;}
  if(maxD == 0.0){maxD_x = maxD_y = maxD_z = maxD_h = -99999;}
  if(verbose)
    CLST << "----------------------------------------------------------------------------------------------------------------------------------------------------------------" << endl;
  else
    CLST << "----------------------------------------------------------------------------------------------------" << endl;
  if(getConcentrationUnitFlag())
    {
      CLST << "+ EXTREME CONCENTRATION (ONLY VALIDATED RESULTS) +" << endl;
      CLST << "^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^" << endl;
      CLST << setw(COLUMN) << "X (m)" << setw(COLUMN) << "Y (m)" << setw(COLUMN) << "Z (m)" << setw(COLUMN) << "H (m)" << setw(COLUMN) << "Conc. (m^3/m^3)" << endl;
      CLST << "----------------------------------------------------------------------------------------------------" << endl;
      CLST << setw(COLUMN) << maxc_x << setw(COLUMN) << maxc_y << setw(COLUMN) << maxc_z << setw(COLUMN) << maxc_h << setw(COLUMN) << maxc << endl;
      CLST << "+ EXTREME CONCENTRATION (INCLUDING NOT VALIDATED RESULTS) +" << endl;
      CLST << "^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^" << endl;
      CLST << setw(COLUMN) << "X (m)" << setw(COLUMN) << "Y (m)" << setw(COLUMN) << "Z (m)" << setw(COLUMN) << "H (m)" << setw(COLUMN) << "Conc. (m^3/m^3)" << endl;
      CLST << "----------------------------------------------------------------------------------------------------" << endl;
      CLST << setw(COLUMN) << maxC_x << setw(COLUMN) << maxC_y << setw(COLUMN) << maxC_z << setw(COLUMN) << maxC_h << setw(COLUMN) << maxC << endl;
      CLST << "+ EXTREME FLUX (ONLY VALIDATED RESULTS) +" << endl;
      CLST << "^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^" << endl;
      CLST << setw(COLUMN) << "X (m)" << setw(COLUMN) << "Y (m)" << setw(COLUMN) << "Z (m)" << setw(COLUMN) << "H (m)" << setw(COLUMN) << "Flux (kg/m^2/s)" << endl;
      CLST << "----------------------------------------------------------------------------------------------------" << endl;
      CLST << setw(COLUMN) << maxd_x << setw(COLUMN) << maxd_y << setw(COLUMN) << maxd_z << setw(COLUMN) << maxd_h << setw(COLUMN) << maxd << endl;
      CLST << "+ EXTREME FLUX (INCLUDING NOT VALIDATED RESULTS) +" << endl;
      CLST << "^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^" << endl;
      CLST << setw(COLUMN) << "X (m)" << setw(COLUMN) << "Y (m)" << setw(COLUMN) << "Z (m)" << setw(COLUMN) << "H (m)" << setw(COLUMN) << "Flux (kg/m^2/s)" << endl;
      CLST << "----------------------------------------------------------------------------------------------------" << endl;
      CLST << setw(COLUMN) << maxD_x << setw(COLUMN) << maxD_y << setw(COLUMN) << maxD_z << setw(COLUMN) << maxD_h << setw(COLUMN) << maxD << endl;
    }
  else
    {
      CLST << "+ EXTREME CONCENTRATION (ONLY VALIDATED RESULTS) +" << endl;
      CLST << "^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^" << endl;
      CLST << setw(COLUMN) << "X (m)" << setw(COLUMN) << "Y (m)" << setw(COLUMN) << "Z (m)" << setw(COLUMN) << "H (m)" << setw(COLUMN) << "Conc. (kg/m^3N)" << endl;
      CLST << "----------------------------------------------------------------------------------------------------" << endl;
      CLST << setw(COLUMN) << maxc_x << setw(COLUMN) << maxc_y << setw(COLUMN) << maxc_z << setw(COLUMN) << maxc_h << setw(COLUMN) << maxc << endl;
      CLST << "+ EXTREME CONCENTRATION (INCLUDING NOT VALIDATED RESULTS) +" << endl;
      CLST << "^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^" << endl;
      CLST << setw(COLUMN) << "X (m)" << setw(COLUMN) << "Y (m)" << setw(COLUMN) << "Z (m)" << setw(COLUMN) << "H (m)" << setw(COLUMN) << "Conc. (kg/m^3N)" << endl;
      CLST << "----------------------------------------------------------------------------------------------------" << endl;
      CLST << setw(COLUMN) << maxC_x << setw(COLUMN) << maxC_y << setw(COLUMN) << maxC_z << setw(COLUMN) << maxC_h << setw(COLUMN) << maxC << endl;
      CLST << "+ EXTREME FLUX (ONLY VALIDATED RESULTS) +" << endl;
      CLST << "^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^" << endl;
      CLST << setw(COLUMN) << "X (m)" << setw(COLUMN) << "Y (m)" << setw(COLUMN) << "Z (m)" << setw(COLUMN) << "H (m)" << setw(COLUMN) << "Flux (kg/m^2/s)" << endl;
      CLST << "----------------------------------------------------------------------------------------------------" << endl;
      CLST << setw(COLUMN) << maxd_x << setw(COLUMN) << maxd_y << setw(COLUMN) << maxd_z << setw(COLUMN) << maxd_h << setw(COLUMN) << maxd << endl;
      CLST << "+ EXTREME FLUX (INCLUDING NOT VALIDATED RESULTS) +" << endl;
      CLST << "^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^" << endl;
      CLST << setw(COLUMN) << "X (m)" << setw(COLUMN) << "Y (m)" << setw(COLUMN) << "Z (m)" << setw(COLUMN) << "H (m)" << setw(COLUMN) << "Flux (kg/m^2/s)" << endl;
      CLST << "----------------------------------------------------------------------------------------------------" << endl;
      CLST << setw(COLUMN) << maxD_x << setw(COLUMN) << maxD_y << setw(COLUMN) << maxD_z << setw(COLUMN) << maxD_h << setw(COLUMN) << maxD << endl;
    }
  head(); CLST << endl;
}

void Rcp::lists(int dy_code, int hr_code, bool verbose)
{
  ML_TRACE("Rcp::lists(int dy_code, int hr_code, bool verbose)");

  CLST << endl;
  CLST << endl;
  CLST << endl;
  CLST << endl;
  CLST << "<< Evaluated concentrations at user defined receptor coordinates in " << "( " << dy_code << ", " << hr_code << ") time class. >>" << endl;
  if(verbose)
    {
      CLST << endl;
      CLST << "+ RECEPTOR THREE DIMENSIONAL COORDINATES, ALTITUDES, CONCENTRATIONS, DEPOSITION FLUXES AND VALIDATION FLAGS +" << endl;
      CLST << "^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^" << endl;
      if(getConcentrationUnitFlag())
        CLST << setw(COLUMN) << "X (m)" << setw(COLUMN) << "Y (m)" << setw(COLUMN) << "Z (m)" << setw(COLUMN) << "H (m)" << setw(COLUMN) << "Conc. (m^3/m^3)" << setw(COLUMN) << "Flux (kg/m^2/s)" << setw(COLUMN) << "Validated (0-1)" << setw(COLUMN) << "DESCRIPTION" << endl;
      else
        CLST << setw(COLUMN) << "X (m)" << setw(COLUMN) << "Y (m)" << setw(COLUMN) << "Z (m)" << setw(COLUMN) << "H (m)" << setw(COLUMN) << "Conc. (kg/m^3N)" << setw(COLUMN) << "Flux (kg/m^2/s)" << setw(COLUMN) << "Validated (0-1)" << setw(COLUMN) << "DESCRIPTION" << endl;
      CLST << "----------------------------------------------------------------------------------------------------------------------------------------------------------------" << endl;
    }
  double maxc = -1.0; double maxc_x, maxc_y, maxc_z, maxc_h;
  double maxd = -1.0; double maxd_x, maxd_y, maxd_z, maxd_h;
  double maxC = -1.0; double maxC_x, maxC_y, maxC_z, maxC_h;
  double maxD = -1.0; double maxD_x, maxD_y, maxD_z, maxD_h;
  head();
  for(int i = 1; i <= getNoElms(); i++)
    {
      const string xx = *x__;
      const string yy = *y__;
      const string zz = *z__;
      const string ss = *cmm;
      const double hh = *hhh;
      const double cc = *va1;
      const double dd = *va2;
      const int    vv = (fabs(*flg) > 1e-6 ? 1 : 0);
      if(verbose) CLST << setw(COLUMN) << xx << setw(COLUMN) << yy << setw(COLUMN) << zz << setw(COLUMN) << hh << setw(COLUMN) << cc << setw(COLUMN) << dd  << setw(COLUMN) << vv << setw(COLUMN) << ss << endl;
      if(vv == 1 && cc > maxc)
        {
          maxc = cc;
          maxc_x = atof (xx.c_str ());
          maxc_y = atof (yy.c_str ());
          maxc_z = atof (zz.c_str ());
          maxc_h = hh;
        }
      if(vv == 1 && dd > maxd)
        {
          maxd = dd;
          maxc_x = atof (xx.c_str ());
          maxc_y = atof (yy.c_str ());
          maxc_z = atof (zz.c_str ());
          maxd_h = hh;
        }
      if(cc > maxC)
        {
          maxC = cc;
          maxc_x = atof (xx.c_str ());
          maxc_y = atof (yy.c_str ());
          maxc_z = atof (zz.c_str ());
          maxC_h = hh;
        }
      if(dd > maxD)
        {
          maxD = dd;
          maxc_x = atof (xx.c_str ());
          maxc_y = atof (yy.c_str ());
          maxc_z = atof (zz.c_str ());
          maxD_h = hh;
        }
      advance();
    }
  if(maxc == 0.0){maxc_x = maxc_y = maxc_z = maxc_h = -99999;}
  if(maxC == 0.0){maxC_x = maxC_y = maxC_z = maxC_h = -99999;}
  if(maxd == 0.0){maxd_x = maxd_y = maxd_z = maxd_h = -99999;}
  if(maxD == 0.0){maxD_x = maxD_y = maxD_z = maxD_h = -99999;}
  if(verbose)
    CLST << "----------------------------------------------------------------------------------------------------------------------------------------------------------------" << endl;
  else
    CLST << "----------------------------------------------------------------------------------------------------" << endl;
  if(getConcentrationUnitFlag())
    {
      CLST << "+ EXTREME CONCENTRATION (ONLY VALIDATED RESULTS) +" << endl;
      CLST << "^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^" << endl;
      CLST << setw(COLUMN) << "X (m)" << setw(COLUMN) << "Y (m)" << setw(COLUMN) << "Z (m)" << setw(COLUMN) << "H (m)" << setw(COLUMN) << "Conc. (m^3/m^3)" << endl;
      CLST << "----------------------------------------------------------------------------------------------------" << endl;
      CLST << setw(COLUMN) << maxc_x << setw(COLUMN) << maxc_y << setw(COLUMN) << maxc_z << setw(COLUMN) << maxc_h << setw(COLUMN) << maxc << endl;
      CLST << "+ EXTREME CONCENTRATION (INCLUDING NOT VALIDATED RESULTS) +" << endl;
      CLST << "^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^" << endl;
      CLST << setw(COLUMN) << "X (m)" << setw(COLUMN) << "Y (m)" << setw(COLUMN) << "Z (m)" << setw(COLUMN) << "H (m)" << setw(COLUMN) << "Conc. (m^3/m^3)" << endl;
      CLST << "----------------------------------------------------------------------------------------------------" << endl;
      CLST << setw(COLUMN) << maxC_x << setw(COLUMN) << maxC_y << setw(COLUMN) << maxC_z << setw(COLUMN) << maxC_h << setw(COLUMN) << maxC << endl;
      CLST << "+ EXTREME FLUX (ONLY VALIDATED RESULTS) +" << endl;
      CLST << "^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^" << endl;
      CLST << setw(COLUMN) << "X (m)" << setw(COLUMN) << "Y (m)" << setw(COLUMN) << "Z (m)" << setw(COLUMN) << "H (m)" << setw(COLUMN) << "Flux (kg/m^2/s)" << endl;
      CLST << "----------------------------------------------------------------------------------------------------" << endl;
      CLST << setw(COLUMN) << maxd_x << setw(COLUMN) << maxd_y << setw(COLUMN) << maxd_z << setw(COLUMN) << maxd_h << setw(COLUMN) << maxd << endl;
      CLST << "+ EXTREME FLUX (INCLUDING NOT VALIDATED RESULTS) +" << endl;
      CLST << "^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^" << endl;
      CLST << setw(COLUMN) << "X (m)" << setw(COLUMN) << "Y (m)" << setw(COLUMN) << "Z (m)" << setw(COLUMN) << "H (m)" << setw(COLUMN) << "Flux (kg/m^2/s)" << endl;
      CLST << "----------------------------------------------------------------------------------------------------" << endl;
      CLST << setw(COLUMN) << maxD_x << setw(COLUMN) << maxD_y << setw(COLUMN) << maxD_z << setw(COLUMN) << maxD_h << setw(COLUMN) << maxD << endl;
    }
  else
    {
      CLST << "+ EXTREME CONCENTRATION (ONLY VALIDATED RESULTS) +" << endl;
      CLST << "^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^" << endl;
      CLST << setw(COLUMN) << "X (m)" << setw(COLUMN) << "Y (m)" << setw(COLUMN) << "Z (m)" << setw(COLUMN) << "H (m)" << setw(COLUMN) << "Conc. (kg/m^3N)" << endl;
      CLST << "----------------------------------------------------------------------------------------------------" << endl;
      CLST << setw(COLUMN) << maxc_x << setw(COLUMN) << maxc_y << setw(COLUMN) << maxc_z << setw(COLUMN) << maxc_h << setw(COLUMN) << maxc << endl;
      CLST << "+ EXTREME CONCENTRATION (INCLUDING NOT VALIDATED RESULTS) +" << endl;
      CLST << "^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^" << endl;
      CLST << setw(COLUMN) << "X (m)" << setw(COLUMN) << "Y (m)" << setw(COLUMN) << "Z (m)" << setw(COLUMN) << "H (m)" << setw(COLUMN) << "Conc. (kg/m^3N)" << endl;
      CLST << "----------------------------------------------------------------------------------------------------" << endl;
      CLST << setw(COLUMN) << maxC_x << setw(COLUMN) << maxC_y << setw(COLUMN) << maxC_z << setw(COLUMN) << maxC_h << setw(COLUMN) << maxC << endl;
      CLST << "+ EXTREME FLUX (ONLY VALIDATED RESULTS) +" << endl;
      CLST << "^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^" << endl;
      CLST << setw(COLUMN) << "X (m)" << setw(COLUMN) << "Y (m)" << setw(COLUMN) << "Z (m)" << setw(COLUMN) << "H (m)" << setw(COLUMN) << "Flux (kg/m^2/s)" << endl;
      CLST << "----------------------------------------------------------------------------------------------------" << endl;
      CLST << setw(COLUMN) << maxd_x << setw(COLUMN) << maxd_y << setw(COLUMN) << maxd_z << setw(COLUMN) << maxd_h << setw(COLUMN) << maxd << endl;
      CLST << "+ EXTREME FLUX (INCLUDING NOT VALIDATED RESULTS) +" << endl;
      CLST << "^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^" << endl;
      CLST << setw(COLUMN) << "X (m)" << setw(COLUMN) << "Y (m)" << setw(COLUMN) << "Z (m)" << setw(COLUMN) << "H (m)" << setw(COLUMN) << "Flux (kg/m^2/s)" << endl;
      CLST << "----------------------------------------------------------------------------------------------------" << endl;
      CLST << setw(COLUMN) << maxD_x << setw(COLUMN) << maxD_y << setw(COLUMN) << maxD_z << setw(COLUMN) << maxD_h << setw(COLUMN) << maxD << endl;
    }
  head(); CLST << endl;
}

void Rcp::setCoor(double  x, double  y, double  z, double  h)
{
  ML_TRACE("Rcp::setCoor");

  (*vec)[0] = x;
  (*vec)[1] = y;
  (*vec)[2] = z;
  *hhh      = h;
}

void Rcp::getCoor(double& x, double& y, double& z, double& h) const
{
  ML_TRACE("Rcp::getCoor");

  x = (*vec)[0];
  y = (*vec)[1];
  z = (*vec)[2];
  h = *hhh;
}
